import express from 'express'
import XmlStream from 'xml-stream'
import fs from 'fs'
import _ from 'lodash'
import axios from 'axios'
import moment from 'moment-timezone'
import getFile from './get-file'
import path from 'path'

const router = express.Router()
if (process.env.NODE_ENV !== 'production') require('dotenv').config()

// Testing Setup
const getLiveToken = true
const testToken = 'wjxnvw5ce2zhc9th3pwg7swa'
const useSFTP = false
const isFirstWrite = false
const isSecondWrite = false
const writeChangesToDB = true
const logChangedItems = false

const appDir = path.dirname(require.main.filename)

let data, response, token, headers, tagobject, tagstring, infusionsoft, client
let tagstoapply = []
let tagstoremove = []

const configureToken = async () => {
  if (getLiveToken) {
    const tokenobject = await client.query(`
      query {
      Infusionsoft(
        id: "${process.env.GRAPHQL_RECORD}"
      ) {
        accesstoken
      }
    }
  `)
    token = tokenobject.Infusionsoft.accesstoken

  } else {
    token = testToken
  }
  headers = {
    'Accept': 'application/json, */*',
    'Authorization': 'Basic ' + token,
    'content-type': 'application/json',
  }
}

// Infusionsoft Default Actions

const addUpdateContact = async (contact) => {
  data = JSON.stringify(contact)
  response = await axios({
    method: 'put',
    headers: headers,
    url: 'https://api.infusionsoft.com/crm/rest/v1/contacts?access_token=' + token,
    data: data
  })
  .catch(e => console.log('This is E', e)) // eslint-disable-line no-console

  return response.data.id
}

const updateContact = async (contact, infusionsoftId) => {
  data = JSON.stringify(contact)

  response = await axios({
    method: 'patch',
    headers: headers,
    url: 'https://api.infusionsoft.com/crm/rest/v1/contacts/' + infusionsoftId + '?access_token=' + token,
    data: data
  })
  .catch(e => console.log('Error', e)) // eslint-disable-line no-console

  return response
}


// Update Tags Not Working!!!


const updateTags = async (tagstoapply, infusionsoftId) => {
  console.log('Updating Tags', tagstoapply)
  tagstring=tagstoapply.toString()
  tagobject = {
    'tagIds': tagstoapply
  }
  data = JSON.stringify(tagobject)
  response = await axios({
    method: 'post',
    headers: headers,
    url: `https://api.infusionsoft.com/crm/rest/v1/contacts/${infusionsoftId}/tags?access_token=${token}&ids=${tagstring}`,
    data: data
  })
  .catch(e => console.log('This is E', e)) // eslint-disable-line no-console

  return response
}

const removeTags = async (tagstoremove, infusionsoftId) => {
  console.log('Removing Tags', tagstoremove, infusionsoftId)
  tagstring=tagstoremove.toString()
  tagobject = {
    'tagIds': tagstoremove
  }
  data = JSON.stringify(tagobject)
  response = await axios({
    method: 'delete',
    headers: headers,
    url: `https://api.infusionsoft.com/crm/rest/v1/contacts/${infusionsoftId}/tags?access_token=${token}&ids=${tagstring}`,
    data: data
  })
  .then(response => {
    return response
  })
  .catch(e => console.log('This is E', e)) // eslint-disable-line no-console
}

// App Specific Actions

const handleAddressBlock = (member) => {
  // Handle Address Logic
  const PrimaryAddress = member.Addresses.AddressSchedule.Billing

  let address1 = null
  let address2 = null
  let addressblock = []

  _.forEach(member.Addresses.Address, (address) => {
    if (address.Country !== 'United States') {
      console.log('Out of US Address', address.Country) // eslint-disable-line no-console
    }
    let region
    if (address.State) {
      region = 'US-' + address.State
    }

    if (address.AddressName === PrimaryAddress) {
      address1 = {
        'country_code': 'USA',
        'field': 'BILLING',
        'line1': address.AddressLine1,
        'line2': address.AddressLine2,
        'locality': address.City,
        'postal_code': address.ZipCode,
        'region': region,
      }
    } else {
      address2 = {
        'country_code': 'USA',
        'field': 'SHIPPING',
        'line1': address.AddressLine1,
        'line2': address.AddressLine2,
        'locality': address.City,
        'postal_code': address.ZipCode,
        'region': region,
      }
    }

    if (address1 && address2) {
      addressblock = [address1, address2]
    } else if (address1 && !address2) {
      addressblock = [address1]
    } else if (!address1 && address2) {
      addressblock = [address2]
    } else {
      addressblock = []
    }
  })
  return addressblock
}

const handleEmailBlock = (member) => {
  let emailblock = []

  // Handle Multiple Emails
  if (member.PrimaryEmail && member.SecondaryEmail) {
    emailblock = [
      {
        'email': member.PrimaryEmail,
        'field': 'EMAIL1'
      },
      {
        'email': member.SecondaryEmail,
        'field': 'EMAIL2'
      },
    ]
  } else if (member.PrimaryEmail && !member.SecondaryEmail) {
    emailblock = [
      {
        'email': member.PrimaryEmail,
        'field': 'EMAIL1'
      }
    ]
  }

  return emailblock
}

const processMemberType = (member, status) => {
  let newMemberType = null
  if (status === 'Active') {
    switch (member.MemberType) {
      case 'Associate Golf':
      case 'Life Time Trans Golf':
      case 'LifeTTransGolf - Platinum':
      case 'TTM Golf':
      case 'VLM Golf':
      case 'Golf':
      case 'Golf-Platinum':
        newMemberType = 'Premier Golf'
        tagstoapply.push(1199)
        tagstoremove.push(1209, 290, 1421, 1415, 1463, 1203, 1205, 1459, 1170, 1487, 1201, 398, 1461, 390)
        break
      case 'Business Designee':
        newMemberType = 'Business Designee'
        tagstoapply.push(1203)
        tagstoremove.push(1475, 290, 1421, 1415, 1463, 1199, 1205, 1459, 1170, 1487, 1201, 398, 1461, 390)
        break
      case 'Diamond Membership':
      case 'Diamond Mem - Platinum':
        newMemberType = 'Diamond Golfer'
        tagstoapply.push(1205)
        tagstoremove.push(1219, 290, 1421, 1415, 1463, 1199, 1203, 1459, 1170, 1487, 1201, 398, 1461, 390)
        break
      case 'Family Club':
      case 'Family Club - (NG) NO GOLF':
      case 'Family Club - Platinum':
      case 'Family Club Platinum (NG)':
      case 'Sport Social':
      case 'Sport Social - Platinum':
        newMemberType = 'Sports Social/Family Club'
        tagstoapply.push(1459)
        tagstoremove.push(1213, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1170, 1487, 1201, 398, 1461, 390)
        break
      case 'Jr. Executive':
      case 'Jr. Executive - Platinum':
        newMemberType = 'JAG'
        tagstoapply.push(1170)
        tagstoremove.push(1211, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1487, 1201, 398, 1461, 390)
        break
      case 'Life Member':
      case 'Life Member - Platinum':
        newMemberType = 'Life Member'
        tagstoapply.push(1487)
        tagstoremove.push(290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1170, 1201, 398, 1461, 390)
        break
      case 'Single Golf':
      case 'Single Golf - Platinum':
        newMemberType = 'Single Golf'
        tagstoapply.push(1201)
        tagstoremove.push(1473, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1170, 1487, 398, 1461, 390)
        break
      case 'Social':
      case 'Social - Platinum':
        newMemberType = 'Social'
        tagstoapply.push(398)
        tagstoremove.push(1217, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1170, 1487, 1201, 1461, 390)
        break
      case 'Student Golf':
        newMemberType = 'Student Golf'
        tagstoapply.push(1461)
        tagstoremove.push(1465, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1170, 1487, 1201, 398, 390)
        break
      case 'Tennis':
      case 'Tennis - Platinum':
        newMemberType = 'Tennis'
        tagstoapply.push(390)
        tagstoremove.push(1215, 290, 1421, 1415, 1463, 1199, 1203, 1205, 1459, 1170, 1487, 1201, 398, 1461)
        break
    }
  } else if (status === 'InActive') {
    switch (member.MemberType) {
      case 'Associate Golf':
      case 'Life Time Trans Golf':
      case 'LifeTTransGolf - Platinum':
      case 'TTM Golf':
      case 'VLM Golf':
      case 'Golf':
      case 'Golf-Platinum':
        newMemberType = 'Premier Golf'
        tagstoremove.push(1199)
        tagstoapply.push(1209, 1463)
        break
      case 'Business Designee':
        newMemberType = 'Business Designee'
        tagstoremove.push(1203)
        tagstoapply.push(1475, 1463)
        break
      case 'Diamond Membership':
      case 'Diamond Mem - Platinum':
        newMemberType = 'Diamond Golfer'
        tagstoremove.push(1205)
        tagstoapply.push(1463, 1219)
        break
      case 'Family Club':
      case 'Family Club - (NG) NO GOLF':
      case 'Family Club - Platinum':
      case 'Family Club Platinum (NG)':
      case 'Sport Social':
      case 'Sport Social - Platinum':
        newMemberType = 'Sports Social/Family Club'
        tagstoremove.push(1459)
        tagstoapply.push(1463, 1213)
        break
      case 'Jr. Executive':
      case 'Jr. Executive - Platinum':
        newMemberType = 'JAG'
        tagstoremove.push(1170)
        tagstoapply.push(1463, 1211)
        break
      case 'Life Member':
      case 'Life Member - Platinum':
        newMemberType = 'Life Member'
        tagstoremove.push(1487)
        tagstoapply.push(1463)
        break
      case 'Single Golf':
      case 'Single Golf - Platinum':
        newMemberType = 'Single Golf'
        tagstoremove.push(1201)
        tagstoapply.push(1473, 1463)
        break
      case 'Social':
      case 'Social - Platinum':
        newMemberType = 'Social'
        tagstoremove.push(398)
        tagstoapply.push(1217)
        break
      case 'Student Golf':
        newMemberType = 'Student Golf'
        tagstoremove.push(1461)
        tagstoapply.push(1465, 1463)
        break
      case 'Tennis':
      case 'Tennis - Platinum':
        newMemberType = 'Tennis'
        tagstoremove.push(390)
        tagstoapply.push(1215)
        break
      case 'Resigned - old':
        tagstoapply.push(1489)
        break
    }
  }

  return newMemberType
}

const processRelationship = async (membernum, coremembernum, relationshipcode) => {
  console.log(`Processing Relationship between ${membernum} and ${coremembernum}`) // eslint-disable-line no-console
  const coreUser = await client.query(`
      query {
      Member(
        memberNumber: "${coremembernum}"
      ) {
      id
        infusionsoftId
        data
      }
    }`
  )
  const thisUser = await client.query(`
      query {
        Member(
          memberNumber: "${membernum}"
        ) {
        id
          infusionsoftId
        }
    }
  `)

  const coreId = parseInt(coreUser.Member.infusionsoftId)
  const userId = parseInt(thisUser.Member.infusionsoftId)

  let relationshiptype

  if (relationshipcode === 'Spouse') {

    updateJoinDate(userId, coreUser.Member.data)
    .catch(e => console.log(e)) // eslint-disable-line no-console

    // Add Married Check Box and Relationship Type
    relationshiptype = 1

    const contact = {
      'custom_fields': [
        {
          'id': 175,
          'content': 'Yes'
        },
      ]
    }

    updateContact(contact, coreId)
    .catch(e => console.log('Error', e)) // eslint-disable-line no-console

    updateContact(contact, userId)
    .catch(e => console.log('Error', e)) // eslint-disable-line no-console

    //Check for other relationship types (currently skipping)
  } else if (relationshipcode === 'Child') {
    relationshiptype = 3
  } else if (relationshipcode === 'Sponsor') {
    relationshiptype = 5
  }

  // Link spouse contact records
  infusionsoft.ContactService
  .linkContacts(coreId, userId, relationshiptype)
}

const updateJoinDate = async (userId, data) => {

  const coreMember = JSON.parse(data)

  if (coreMember.JoinDate && coreMember.JoinDate !== 19000101) {
    const contact = {
      'custom_fields': [
        {
          'id': 61,
          'content': moment(new Date(coreMember.JoinDate)).format()
        },
      ]
    }

    await updateContact(contact, userId)
    .catch(e => console.log('Error', e)) // eslint-disable-line no-console

  }
}

const getReadStream = () => {
  if (useSFTP) {
    const filePath = getFile()
    console.log(filePath) // eslint-disable-line no-console
    const Stream = fs.createReadStream(filePath)
    return Stream
  } else {
    const Stream = fs.createReadStream(`${appDir}/data/Test.xml`)
    console.log('Using Testing Readstream') // eslint-disable-line no-console
    return Stream
  }
}

router.get('/', async (req, res) => {

  await configureToken()

  const readStream = await getReadStream()

  readStream.on('open', function () {

    const xml = new XmlStream(readStream)

    xml.collect('Address')
    xml.on('updateElement: MemberId', async function (item) {
      if (isFirstWrite) {
        let itemString = await JSON.stringify(item)
        itemString = itemString.replace(/(["])/g, '\\$1')
        await client.mutate(`{
        newClient: createMember(
            memberId: "${item.$.ID}"
            memberNumber: "${item.MemberInfo.MemberNumber}"
            coreMemberNumber: "${item.MemberInfo.CoreMemberNumber}"
            data: "${itemString}"
          ) {
          id

          }
        }`
        ).then(response => {
          console.log(`Written User ${item.$.ID} to ${response.newClient.id}`) // eslint-disable-line no-console
        })
        .catch(e => console.log(e)) // eslint-disable-line no-console

      }
      else {

        const clientquery = await client.query(`
        query {
          Member(
            memberId: "${item.$.ID}"
          ) {
            id
            memberNumber
            coreMemberNumber
            infusionsoftId
            memberId
            data
          }
        }
      `)

        const dbentry = (clientquery.Member) ? JSON.parse(clientquery.Member.data) : null

        if ((isSecondWrite || !_.isEqual(dbentry, item))
          // Exclude these member types from transferring
          && item.MemberInfo.MemberType !== 'Banquets'
          && item.MemberInfo.MemberType !== 'Annual Golf'
          && item.MemberInfo.MemberType !== 'Delinquent Write Off'
          && item.MemberInfo.MemberType !== 'Dreyfuss LCFCC Emp'
          && item.MemberInfo.MemberType !== 'Golf Guest'
          && item.MemberInfo.MemberType !== 'Guest'
          && item.MemberInfo.MemberType !== 'House Accounts'
          && item.MemberInfo.MemberType !== 'LOA - Regular'
          && item.MemberInfo.MemberType !== 'LOA - Tennis'
          && item.MemberInfo.MemberType !== 'Member W/O Dues'
          && item.MemberInfo.MemberType !== 'Reciprical Clubs'
          && item.MemberInfo.MemberType !== 'Summer Social'
          && item.MemberInfo.MemberType !== 'TTM Social'
          && item.MemberInfo.RelationShipCode !== 'Child'
        ) {

          if (logChangedItems) {
            console.log('CHANGE DETECTED', dbentry, item) // eslint-disable-line no-console
          }

          const member = item.MemberInfo
          const addressblock = handleAddressBlock(member)
          const emailblock = handleEmailBlock(member)

          if (member.Status === 'Active') {
            // Member is Active
            // Apply 'Do Not Send - Club Member'
            tagstoapply.push(368)

            const newMemberType = processMemberType(member, 'Active')

            console.log(moment(member.Birthday, 'YYYYMMDD').format())
            // Create Contact Object
            const contact = {
              'duplicate_option': 'Email',
              'anniversary': member.Anniversary ? moment(member.Anniversary, 'YYYYMMDD').format() : null,
              'addresses': addressblock,
              'birthday': member.Birthday ? moment(member.Birthday, 'YYYYMMDD').format() : null,
              'contact_type': 'Customer',
              'custom_fields': [
                {
                  'id': 97,
                  'content': member.MemberNumber
                },
                {
                  'id': 133,
                  'content': newMemberType
                },
                {
                  'id': 61,
                  'content': member.JoinDate
                },
                {
                  'id': 169,
                  'content': member.Gender
                },
              ],
              'email_addresses': emailblock,
              'family_name': member.LastName,
              'given_name': member.FirstName,
              'last_updated': moment(new Date()).format(),
              'middle_name': member.MiddleName,
              'opt_in_reason': 'Country Club Member - Opted in on site and contract',
              'phone_numbers': [
                {
                  'field': 'PHONE1',
                  'number': member.CellPhone,
                  'type': 'Mobile'
                },
                {
                  'field': 'PHONE3',
                  'number': member.BusinessPhone,
                  'type': 'Work'
                },
                {
                  'field': 'PHONE2',
                  'number': member.HomePhone,
                  'type': 'Home'
                }
              ],
              'prefix': member.Prefix,
              'source_type': 'API',
              'suffix': member.Suffix,
            }
            // console.log(contact) // eslint-disable-line no-console

            // Update Contact
            const infusionsoftId = await
              addUpdateContact(contact)

            console.log('Tags to Apply: ', tagstoapply)
            await updateTags(tagstoapply, infusionsoftId)
            .catch(e => console.log('Error', e)) // eslint-disable-line no-console

            console.log('Tags to Remove: ', tagstoremove)
            await removeTags(tagstoremove, infusionsoftId)
            .catch(e => console.log('Error', e)) // eslint-disable-line no-console


            if (writeChangesToDB) {
              if (clientquery.Member && clientquery.Member.id) {
                let itemString = await JSON.stringify(item)
                itemString = itemString.replace(/(["])/g, '\\$1')
                await client.mutate(`{
                    updatedMember: updateMember(
                    id: "${clientquery.Member.id}"
                    memberId: "${item.$.ID}"
                    memberNumber: "${item.MemberInfo.MemberNumber}"
                    coreMemberNumber: "${item.MemberInfo.CoreMemberNumber}"
                    infusionsoftId: "${infusionsoftId}"
                    data: "${itemString}"
                  ) {
                  id
                  }
                }`
                ).then(response => {
                  console.log(`Updated User ${item.$.ID} at ${response.updatedMember.id}`) // eslint-disable-line no-console
                })
                .catch(e => console.log(e)) // eslint-disable-line no-console
              } else {
                let itemString = await JSON.stringify(item)
                itemString = itemString.replace(/(["])/g, '\\$1')
                await client.mutate(`{
                    newClient: createMember(
                    memberId: "${item.$.ID}"
                    memberNumber: "${item.MemberInfo.MemberNumber}"
                    coreMemberNumber: "${item.MemberInfo.CoreMemberNumber}"
                    infusionsoftId: "${infusionsoftId}"
                    data: "${itemString}"
                  ) {
                  id
        
                  }
                }`
                ).then(response => {
                  console.log(`Written User ${item.$.ID} to ${response.newClient.id}`)// eslint-disable-line no-console
                })
                .catch(e => console.log(e)) // eslint-disable-line no-console
              }
            }

            // Do Spouse Logic
            if (member.MemberNumber !== member.CoreMemberNumber) {
              processRelationship(member.MemberNumber, member.CoreMemberNumber, member.RelationShipCode)
            }
          }
          else if (member.Status === 'InActive' && dbentry.MemberInfo.Status === 'Active') {
            tagstoapply.push(290, 1421, 1415)
            tagstoremove.push(368)

            const newMemberType = await processMemberType(member, 'InActive')

            // Create Contact Object
            const contact = {
              'duplicate_option': 'Email',
              'anniversary': member.Anniversary ? moment(member.Anniversary, 'YYYYMMDD').format() : null,
              'addresses': addressblock,
              'birthday': member.Birthday ? moment(member.Birthday, 'YYYYMMDD').format() : null,
              'contact_type': 'Customer',
              'custom_fields': [
                {
                  'id': 97,
                  'content': member.MemberNumber
                },
                {
                  'id': 133,
                  'content': newMemberType
                },
                {
                  'id': 61,
                  'content': member.JoinDate
                },
                {
                  'id': 63,
                  'content': moment(new Date()).format()
                },
                {
                  'id': 69,
                  'content': member.Gender
                },
              ],
              'email_addresses': emailblock,
              'family_name': member.LastName,
              'given_name': member.FirstName,
              'last_updated': moment(new Date()).format(),
              'middle_name': member.MiddleName,
              'opt_in_reason': 'Country Club Member - Opted in on site and contract',
              // "owner_id": 0,
              'phone_numbers': [
                {
                  'field': 'PHONE1',
                  'number': member.CellPhone,
                  'type': 'Mobile'
                },
                {
                  'field': 'PHONE2',
                  'number': member.BusinessPhone,
                  'type': 'Work'
                },
                {
                  'field': 'PHONE3',
                  'number': member.HomePhone,
                  'type': 'Home'
                }
              ],
              'prefix': member.Prefix,
              'source_type': 'API',
              'suffix': member.Suffix,
            }

            // console.log(contact) // eslint-disable-line no-console

            const infusionsoftId = await addUpdateContact(contact)
            console.log('Infusionsoft ID: ', infusionsoftId) // eslint-disable-line no-console

            console.log('Tags to Apply: ', tagstoapply)
            await updateTags(tagstoapply, infusionsoftId)
            .catch(e => console.log('Error', e)) // eslint-disable-line no-console

            console.log('Tags to Remove: ', tagstoremove)
            await removeTags(tagstoremove, infusionsoftId)
            .catch(e => console.log('Error', e)) // eslint-disable-line no-console

            if (writeChangesToDB) {
              if (clientquery.Member && clientquery.Member.id) {
                let itemString = await JSON.stringify(item)
                itemString = itemString.replace(/(["])/g, '\\$1')
                await client.mutate(`{
                    updatedMember: updateMember(
                    id: "${clientquery.Member.id}"
                    memberId: "${item.$.ID}"
                    memberNumber: "${item.MemberInfo.MemberNumber}"
                    coreMemberNumber: "${item.MemberInfo.CoreMemberNumber}"
                    infusionsoftId: "${infusionsoftId}"
                    data: "${itemString}"
                  ) {
                  id
                  }
                }`
                ).then(response => {
                  console.log(`Updated User ${item.$.ID} at ${response.updatedMember.id}`) // eslint-disable-line no-console
                })
                .catch(e => console.log(e)) // eslint-disable-line no-console
              } else {
                let itemString = await JSON.stringify(item)
                itemString = itemString.replace(/(["])/g, '\\$1')
                await client.mutate(`{
                    newClient: createMember(
                    memberId: "${item.$.ID}"
                    memberNumber: "${item.MemberInfo.MemberNumber}"
                    coreMemberNumber: "${item.MemberInfo.CoreMemberNumber}"
                    infusionsoftId: "${infusionsoftId}"
                    data: "${itemString}"
                  ) {
                  id
        
                  }
                }`
                ).then(response => {
                  console.log(`Written User ${item.$.ID} to ${response.newClient.id}`)// eslint-disable-line no-console
                })
                .catch(e => console.log(e)) // eslint-disable-line no-console
              }
            }
          }
        }
      }
    })

    xml.on('end', function () {
      console.log('DONE')// eslint-disable-line no-console
    })
  })

  readStream.on('error', function (err) {
    console.log(err) // eslint-disable-line no-console
    res.end(err)
  })

  res.send('Finished Processing')

})

export default (c, i) => {
  client = c
  infusionsoft = i
  return router
}
